/*

ti pa
00.0 = void
01.3 = sky

24.3 25.3 26.3 27.3
2C.3 33.3 33.3 2F.3
34.3 33.3 33.3 37.3
3C.3 3D.3 3E.3 3F.3 = big ball

38.3 39.3 = big branch

2D.3 2E.3
35.3 36.3 = small ball

3A.3 3B.3 = small branch

61.0 = horizontal pipe
60.1 = pipe left edge
62.1 = pipe right edge

2B.2 = inside background
23.2 = inside shadow

68.0 {69.0} 6A.0
70.0 {71.0} 72.0 = pillar

83.0 = ladder

C4.1 = spike up

51.0 52.0 53.0 = wily ornament
10.0 = wily door



*/
static void BombmanAscii(Screen& s)
{
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = s.outmap[y*16+x].tile;
            unsigned char p = s.outmap[y*16+x].palette;
            
            char ch = '?';
            /* Classify types.
             *
             *
             *
             */
            switch(c)
            {
                // sky:
                case 0x01: // sky
                case 0x24 ... 0x27:
                case 0x2C: case 0x33: case 0x2F:
                case 0x34: case 0x37:
                case 0x3C ... 0x3F: // big tree
                case 0x38 ... 0x39: // big branch
                case 0x2D ... 0x2E:
                case 0x35 ... 0x36: // small tree
                case 0x3A ... 0x3B: // small branch
                case 0x00: // void
                    ch = '.';
                    break;
                
                case 0x60 ... 0x62:
                    ch = '=';
                    break;
                case 0x68 ... 0x6A:
                case 0x70 ... 0x72:
                case 0x51 ... 0x53:
                    ch = '#';
                    break;
                case 0x23: case 0x2B:
                    ch = ' '; // inside
                    break;
                case 0x83:
                    ch = Ascii_Ladder;
                    break;
                case 0xC4:
                    ch = Ascii_Spike;
                    break;
                    
                case 0x10:
                    ch = Ascii_WilyDoor;
                    break;
                
                // There are no gutsblocks in Bombman level.
            }
            
            s.ascii[y*32+x] = ch;
            s.ascii[y*32+x+16] = ' ';
            //printf("%02X",c);      
        }
}

#include "treegen.hh"

namespace BombUtil
{
    bool isFree(unsigned size, unsigned x, unsigned y,
                const unsigned char* screen,
                const char* ground)
    {
        bool hitsGround = false;

        unsigned FreeSpace=0, HitSpace=0;
        for(unsigned iy = y; iy < y+size; ++iy)
            for(unsigned ix = x; ix < x+size; ++ix)
            {
                if(screen[ix+16*iy] != 0) return false;
                if(ground[ix+32*iy] != '.') ++HitSpace; else ++FreeSpace;
            }
        if(HitSpace > FreeSpace) return false;

        const unsigned bx = x+size/2-1;
        for(unsigned iy = y+size-1; iy < 16; ++iy)
        {
            unsigned groundHits = 0;
            for(unsigned ix = bx; ix < bx+2; ++ix)
            {
                if(screen[ix+16*iy] != 0) return false;
                if(ground[ix+32*iy] != '.') ++groundHits;
                if(ground[ix+32*iy] == '^') return false;
            }
            if(groundHits == 2) hitsGround = true;
        }

        return hitsGround;
    }

    void CreateTrees(unsigned char dest[16*16], const char ground[32*16])
	{
        const unsigned MaxTrees = 8, MaxTries = 100;
        const unsigned LowerSideOffset = 2;

		static const unsigned char BigBall[4][4] =
		{
			{ 0x24, 0x25, 0x26, 0x27 },
			{ 0x2C, 0x33, 0x33, 0x2F },
			{ 0x34, 0x33, 0x33, 0x37 },
			{ 0x3C, 0x3D, 0x3E, 0x3F }
		};
		static const unsigned char BigBranch[2] = { 0x38, 0x39 };
		static const unsigned char SmallBall[2][2] =
		{
			{ 0x2D, 0x2E },
			{ 0x35, 0x36 }
		};
		static const unsigned char SmallBranch[2] = { 0x3A, 0x3B };

		unsigned tries = 0;
		for(unsigned treeCount = 0; treeCount < MaxTrees; ++treeCount)
		{
			while(true)
			{
				if(++tries > MaxTries) return;
	 
				bool big = std::rand() % 3; // Big trees more probable than small ones
				if(big)
				{
					unsigned x = std::rand() % (16 - 4);
					unsigned y = std::rand() % (16 - 4 - 2);
					if(!isFree(sizeof(BigBall[0]), x, y, dest, ground)) continue;
					CreateTree<4,4,2> (BigBall, BigBranch, dest, x, y,
					           16-y-4);
					break;
				}
				else
				{
					unsigned x = std::rand() % (16 - 2);
					unsigned y = std::rand() % (16 - 2 - 2);
					if(!isFree(sizeof(SmallBall[0]), x, y, dest, ground)) continue;
					CreateTree<2,2,2> (SmallBall, SmallBranch, dest, x, y,
					           16-y-2);
					break;
				}
			}
		}
	}
}

static void BombmanChar(Screen& s)
{
    unsigned randseed = 0;
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
            randseed += s.ascii[y*32+x];
    std::srand(randseed);
    
    unsigned char decorations[16*16] = { 0 };
    BombUtil::CreateTrees(decorations, s.ascii);
    
    // Convert to tiles
    for(unsigned y=0; y<16; ++y) 
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c   = 0;
            unsigned char pal = 0;
            
            switch(s.ascii[y*32+x])
            {
                case '.':
                {
                    // background
                    char deco = decorations[y*16+x];
                    c   = deco ? deco : 0x01;
                    pal = 3;
                    break;
                }
                case '#':
                {
                    bool left  = x>0  && s.ascii[y*32+(x-1)]=='#';
                    bool right = x<15 && s.ascii[y*32+(x+1)]=='#';
                    bool up    = y==0 || s.ascii[(y-1)*32+x]!='.';
                    // walls
                    c   = 0x71;
                    if(!up) c -= 8;
                    if(!left && right) c -= 1;
                    else if(!right &&left) c += 1;
                    pal = 0;
                    break;
                }
                case ' ':
                {
                    bool up  = y==0 || s.ascii[(y-1)*32+x]==' ';
                    c   = up ? 0x2B : 0x23;
                    pal = 2;
                    // inside
                    break;
                }
                case '=':
                {
                    bool left  = x>0  && s.ascii[y*32+(x-1)]!='.';
                    bool right = x<15 && s.ascii[y*32+(x+1)]!='.';
                    // pipes
                    c   = 0x61;
                    pal = 0;
                    if(!left && right)     { c -= 1; pal=1; }
                    else if(!right &&left) { c += 1; pal=1; }
                    break;
                }
                case Ascii_Ladder:
                    c   = 0x82; // ladder
                    pal = 0x00;
                    break;
                case Ascii_Spike:
                    c   = 0xC4; // spike
                    pal = 0x01;
                    break;
                case Ascii_WilyDoor:
                    c   = 0x10;
                    pal = 0x00;
                    break;
                default:
                    printf("? '%c'\n", s.ascii[y*32+x]);
            }
            
            s.outmap[y*16+x].tile    = c;
            s.outmap[y*16+x].palette = pal;
        }
}

static Level Bombman =
{
	2, // level number
	"Bombman",

	// screens:
	{
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"====####========"   "                "
		"....####........"   "                "
		"====####========"   "                "
		"....####........"   "                "
		, 0 // monster graphics index
		,LevelBegin| Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "          j     "
		"........####...."   "                "
		"........####...."   "      j      j  "
		"....====####==.."   "                "
		"....====####...."   "   j           j"
		"========####===="   "                "
		"........####===="   "                "
		"========####===="   "                "
		"........####...."   "                "
		"========####===="   "                "
		"........####...."   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"====####========"   "                "
		"....####........"   "                "
		"====####========"   "                "
		"....####........"   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"..##......##...."   "                "
		"..##......##...."   "                "
		"==##....==##...."   "                "
		"==##......##...."   "                "
		"==##..====##..=="   "                "
		"..##..====##...."   "                "
		"==##......##..=="   "     B       B  "
		"..##......##...."   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"..##......##...."   "                "
		"..##......##...."   "                "
		"==##....==##...."   "                "
		"==##......##...."   "                "
		"==##..====##..=="   "                "
		"..##..====##...."   "                "
		"==##......##..=="   "     B       B  "
		"..##......##...."   "                "
		, 0 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		".............##."   "                "
		"....##...====##."   "                "
		"....##.........."   "                "
		"..==##.........."   "                "
		"....##.........."   "                "
		"====##=======##="   "          v     "
		"....##=======##."   "                "
		"====##=======##="   "                "
		"....##=======##."   "                "
		, 0 // monster graphics index
		,Right},
		{
		"..............|="   "                "
		"..............|="   "                "
		"..............|="   "                "
		"..............|="   "                "
		".......======##="   "  ee            "
		"..==.........##."   "                "
		"..........===##="   "                "
		"......=....==##="   "                "
		"............=##="   "                "
		".............##."   "                "
		".....=.......##="   "            P   "
		"...........==##="   "                "
		"=============##="   "v     v         "
		".............##."   "                "
		"=============##="   "                "
		".............##="   "                "
		, 0 // monster graphics index
		,Up},
		{
		"..|........====="   "                "
		"..|.........===="   "                "
		"..|............="   "                "
		"..|...........=="   "                "
		"..==##==|=.....="   "                "
		"....##..|=.....="   "      >         "
		"....##..|=##...="   "                "
		"....##..|=##...."   "      ^         "
		"....##..|.##==|="   "                "
		"....##..|=##==|."   "      >         "
		"....##..|....=|="   "                "
		"....##..|....=|="   "      ^     E   "
		"........==##==|="   "                "
		"^^^^^^^^..##..|."   "                "
		"==========##==|="   "                "
		"..........##..|="   "                "
		, 1 // monster graphics index
		,Up},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"..|=====####===="   "                "
		"..|=====####...."   "                "
		"..|=====####===="   "                "
		"..|=====####...."   "                "
		, 1 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "G               "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"====####========"   "                "
		"....####........"   "                "
		"====####========"   "                "
		"....####........"   "                "
		, 1 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"..##......##...."   "                "
		"..##......##...."   "         <      "
		"==##....==##...."   "        s       "
		"==##......##...."   "                "
		"==##..====##..=="   "                "
		"..##..====##...."   "                "
		"==##......##..=="   "                "
		"..##......##...."   "                "
		, 1 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"..##......##...."   "                "
		"..##......##...."   " <       <      "
		"==##....==##...."   "                "
		"==##......##...."   "                "
		"==##..====##..=="   "                "
		"..##..====##...."   "                "
		"==##......##..=="   "                "
		"..##......##...."   "                "
		, 1 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"..|##=======...."   "                "
		"..|##=======...."   "                "
		"..|##..........."   "                "
		"..|##..........."   "            f   "
		"..|##..=======##"   "                "
		"..|##..=======##"   "                "
		"..|##..........."   "                "
		"..|##..........."   "                "
		"===##=========##"   "                "
		"...##=========##"   "                "
		"===##.........##"   "                "
		"...##.........##"   "                "
		, 1 // monster graphics index
		,Right},
		{
		"..............|="   "                "
		"..............|="   "                "
		"..............|="   "                "
		"..............|="   "                "
		".......======##="   "  f             "
		"..==.........##."   "                "
		"..........===##="   "                "
		"......=....==##="   "                "
		"............=##="   "                "
		".............##."   "                "
		".....=.......##="   "                "
		"...........==##="   "          f     "
		"=============##="   "                "
		".............##."   "                "
		"=============##="   "                "
		".............##="   "                "
		, 1 // monster graphics index
		,Up},
		{
		"..|............."   "                "
		"..|............."   "                "
		"..|............."   "                "
		"..|............."   "                "
		"..|............."   "                "
		"..|..=...=...=|="   "                "
		"..|...........|="   "                "
		"..|...........|="   "                "
		"..|...........|."   "               s"
		"..|...........|="   "                "
		"..|...........|."   "                "
		"..|...........|="   "                "
		"......========|="   "                "
		"^^^^^^........|="   "                "
		"==============|."   "                "
		"..............|."   "                "
		, 1 // monster graphics index
		,Up| RestartPointA},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"........=...=..."   "                "
		"................"   "               u"
		"................"   "                "
		".......=........"   "                "
		"................"   "                "
		"..|##..........."   "                "
		"..|##^^^^^^^^^^^"   "                "
		"..|##==========="   "                "
		"..|##==========="   "                "
		, 2 // monster graphics index
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"====####========"   "                "
		"....####........"   "                "
		"====####========"   "                "
		"....####........"   "                "
		, 2 // monster graphics index
		,Right},
		{
		"........##......"   "                "
		"........##......"   "                "
		".....##.##......"   "                "
		".....##.##......"   "                "
		"..|==##........."   "             G  "
		"..|==##........."   "                "
		"..|..##.##......"   "                "
		"..|==##.##......"   "               U"
		"..|..##.##======"   "                "
		"..|==##.##======"   "                "
		"..|==##........."   "                "
		"..|==##........."   "                "
		"=====##========="   "                "
		".....##........."   "                "
		"=====##========="   "                "
		".....##........."   "           u    "
		, 2 // monster graphics index
		,Right},
		{
		"......====####=="   "                "
		"......====####=="   "                "
		"..........####=="   "                "
		"......====####.."   "                "
		"..........####=="   "                "
		"......====####=="   "                "
		"......====####=="   "                "
		"......====####=="   "                "
		"==========####.."   "                "
		"==========####.."   "                "
		"................"   "                "
		"................"   "                "
		"==========####=="   "                "
		"..........####=="   "                "
		"==========####=="   "                "
		"..........####.."   "                "
		, 2 // monster graphics index
		,Right},
		{
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		"..............DD"   "              SS"
		"..............DD"   "              SS"
		"..............DD"   "             GSS"
		"..............DD"   "              SS"
		"..........##===="   "                "
		"..........##===="   "                "
		"==========##===="   "                "
		"==========##===="   "                "
		"==========##===="   "                "
		"..........##===="   "                "
		, 2 // monster graphics index
		,Shutter},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"==========##...."   "                "
		"==========##...."   "                "
		"          ##...."   "                "
		"          ##...."   "                "
		"          ##...."   "                "
		"          ##...."   "                "
		"====##  | ##...."   "                "
		"====##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		, 3 // monster graphics index
		,Down| RestartPointB},
		{
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "      h         "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "         h      "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "      h         "
		"....##  | ##...."   "                "
		"....##  | ##...."   "         h      "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		, 3 // monster graphics index
		,Down},
		{
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "      h         "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "         h      "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		"....##  | ##...."   "      h         "
		"....##  | ##...."   "                "
		"....##  | ##...."   "         h      "
		"....##  | ##...."   "                "
		"....##  | ##...."   "                "
		, 3 // monster graphics index
		,Down},
		{
		"#===##    ##===#"   "                "
		"#===##    ##===#"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#===##====##===#"   "                "
		"#===##====##===#"   "                "
		"#===##====##===#"   "                "
		"#===##====##===#"   "                "
		, 4 // monster graphics index
		,End},
	},

	// palette
	{
		15, 32, 16, 17, 
		15, 32, 16, 33, 
		15, 41, 25,  7, 
		15, 39,  6, 33, 

		15, 44, 17, 15, 
		15, 32, 56, 15, 
		15, 48, 21, 15, 
		15, 48, 17, 15, 

		15, 32, 16, 17, 
		15, 32, 16, 33, 
		15, 32, 16,  0, 
		15, 39,  6, 33, 
	},

	// after doors palette
	{
		15, 48, 16,  6, 
		15, 27, 11,  0, 
		15, 39, 22,  6, 
		15, 38, 22,  6, 

	},

	// actives
	{
	},

	// enemies
	{
	},

	// level gfx (id, count) (max 16); id is block*4+bank
	{
		{ 0x40, 8 }, { 0xBA, 1 }, { 0x7E, 2 }, { 0x7A, 2 }, 
		{ 0xE2, 1 }, { 0xD8, 1 }, { 0x5E, 1 }, { 0xC0, 8 }, 
		{ 0x81, 8 }, 
	},

	// monster gfx tables (max 11) (5 tiles, 2*3bytepalette)
	{
		{ { 0xBA, 0x7E, 0xAE, 0x7A, 0x7E, }, { 48,21,15, 48,17,15,} },
		{ { 0x96, 0x9A, 0x9E, 0x62, 0x72, }, { 48,42,15, 48,38,15,} },
		{ { 0x96, 0x9A, 0x9E, 0x7A, 0x72, }, { 48,42,15, 48,38,15,} },
		{ { 0x6E, 0xCA, 0x76, 0x76, 0x7E, }, { 48,21,15, 48,21,15,} },
		{ { 0x80, 0x84, 0x88, 0x8C, 0x4A, }, { 39,21,15, 48,38,15,} },
	},

	0x20, // Arrival input

	BombmanAscii, BombmanChar
};
