#ifndef bqtRockEditDEFsHH
#define bqtRockEditDEFsHH

/*
    C-I-B-F-E-G CIBFEG
    
    KSITTELEMTT?
        
        Table C2D4: (OK)stage restart points A
        Table C2E0: (OK)stage restart points B
        Table C2EB: (OK)mist levelit alkaa
        Table C2F7: (OK)restart point A screen (?)
        Table C303: (OK)restart point B screen (?)
        
        Table 059380: (OK)keys that are automatically held when arriving in boss's room
                     0=left, $20=up, $10=down
       
        Table 059F1D: (OK)palettes for after-doors
                     16 bytes each stage
        
        Table 0593A0: (OK)order positions at start (0,0,0,0,0,0, 14,7,9,9,22,7)
        Table 0593AC: (OK)order positions at checkp1 (5,1,3,4,7,1, 15,12,13,12,12,12)
        Table 0593B8: (OK)order positions at checkp2 (12,5,5,7,18,5, 20,16,13,16,25,525)
        Table 059F02: (OK) screen with 1st door (19,19,19,15,19,14, -1,-1,-1,-1)
        Table 059EF7: (OK) screen with 2nd door (22,22,-1,18,-1,17,-1,-1,-1,34,32)
                      usually boss room-1, but not always there's a shutter
        
        Table CE31: Tile that replaces shutter at second door (if zero, use same as 1st)
        Table CE3D: Tile that replaces shutter at first door
        Table CE49: Screen where second shutter is (always the last room of scene)
        
        Table ED91: Something for bosses
        Table ED9A: Boss initial X coord ;usually C0; for Wily2 it's B0.
        Table EDA3: Boss initial Y coord ;Uusally B4, for Gutsman it's B0.
                     ; Boss coordinates are not listed for Wily4.

        ; Note: for Wily2 rematches, X coord is at 1DC13 lda #$40
        
        Table C51A: Teleport Y coord at start    ;usually 94,B4,C4,64,54...
        Table C526: Teleport Y coord at restartA
        Table C532: Teleport Y coord at restartB
        
        
*/

enum
{
    Up      = 2,
    Down    = 4,
    Shutter = 8,
    LevelBegin    = 16,
    RestartPointA = 32,
    RestartPointB = 64,
    Right   = 1,
    End     = 1,
    NoRooms = 0
};

// Routine $C39C: load the page of the level

/* THESE ADDRESSES ARE RELATIVE TO LEVEL START PAGE.
 * THEREFORE, DON'T USE $8000..$BFFF, BUT USE $0000..$3FFF.
 */

// In each page, these settings are constant:
static const unsigned Addr_Blocks   = 0x0000; // Max blocks: 192 (shared)
static const unsigned Addr_BlockPals= 0x0300; // Max blocks: 192 (shared)
static const unsigned Addr_Rooms    = 0x03C0; // Max rooms: 33   (shared)
/* "Rooms": Storage for rooms.
 * "Addr_Pointers" will point here.
 */

static const unsigned Addr_Screens  = 0x0C00; // Max screens: 48 (shared)
/* "Screens": For each 48 different screens, contains an index to "Pointers".
 * It converts a "screen num" into a "room num".
 */

static const unsigned Addr_Pointers = 0x0C30; // Max rooms: 32.  (shared)
/* "Pointers": Points to "Rooms". Indexed by an "screen".
 *
 * It converts a "room num" into a "room data pointer".
 */

static const unsigned Addr_Layout   = 0x0C70; // Max screens: 48. (shared)
/* "Layout": It tells how the stage scrolls.
 */

static const unsigned Addr_Palette1 = 0x0CA0;
static const unsigned Addr_Palette2 = 0x0CD0;

// Map1 ?table (number, followed by 2 bytes each)
// First byte:  A to SwitchChr (2 low bits is page, 6 high is block number)
// Second byte: the number of 256-byte entities to copy.
// This table is consumed entirely by a single call of WriteChr.
// This table seems to define which graphics the level
// is constructed from.
static const unsigned Addr_Tiles1   = 0x0D00;
// Same as Tiles1, but for the Wily version
static const unsigned Addr_Tiles2   = 0x0D20;

// Something common to both. Unit: 1 byte. Indexed by screen id.
static const unsigned Addr_MonsterGraphicsIndex = 0x0D40;

// Something common to both. Unit: 2 bytes
// In 0D80-table, either one of the bytes 
// is a (block index*4)+(page number).
//
// One 32-byte entity will be copied from the block data into $382.

// Indexed by (value from D40) PLUS something*something
//
// Seems to contain 11-byte records,
// where 5 bytes are object numbers (or similar) and 6 bytes are palette.

static const unsigned Addr_MonsterGraphicsTable = 0x0D80;

static const unsigned Addr_Actives1 = 0x0E00;
static const unsigned Addr_Actives2 = 0x0EA0;
static const unsigned Addr_ActivesEnd=0x0F00; // begin something?
static const unsigned Addr_Tiles3c   =0x0F80; // Something common to both, also F70.
static const unsigned Addr_GFXstart  =0x1000;
  // E00: 1 byte: number of objects (map1) Including terminator
  //      6 bytes per object. Terminator = 2nd byte is $FF
  //      Max objects: 29
  //  0: type
  //       2=gutsblock, 3=door_right, 1=door_left, 0=appearing block
  //  1: screen #
  //  2: x1 (in)
  //  3: y1 (in)
  //  4: x2 (out)
  //  5: y2 (out)
  // EA0: 1 byte: number of objects (map2) Including terminator
  //      6 bytes per object. Terminator = 2nd byte is $FF
  //      Max objects: 13
  // F00: ?
  
  //1000: gfx tiles actually begin here.

static const unsigned Addr_EnemyPointers = 0x06A452; //C,I,B,F,E,G,1,2,3,4,? point to A468+.
static /***/ unsigned Addr_EnemyData     = 0x06A468; // room for 419 enemies (total).
static const unsigned Addr_EnemyEnd      = 0x06AB00;

/* Address of the 10-byte table which lists the offsets of
 * each level's order table, relative to the page's Addr_Screens.
 */
static const unsigned Addr_FirstScreenTable = 0xC2EB;
static const unsigned Addr_FirstOrderTable  = 0x0593A0;
static const unsigned Addr_FirstEnemyTable  = 0x0593C3;
/* When you die after this screen, you will be returned to the restart point */
static const unsigned Addr_CheckPointsScreenTable1 = 0xC2D4;
static const unsigned Addr_CheckPointsScreenTable2 = 0xC2E0;
/* The restart point has this particular screen number */
static const unsigned Addr_RestartPointsScreenTable1 = 0xC2EB+12;
static const unsigned Addr_RestartPointsScreenTable2 = 0xC2EB+24;
/* And this particular order number */
static const unsigned Addr_RestartPointsOrderTable1 = 0x0593A0+12;
static const unsigned Addr_RestartPointsOrderTable2 = 0x0593A0+24;
/* And enemies are loaded up to this index */
static const unsigned Addr_RestartEnemyPointsTable1 = 0x0593C3+12;
static const unsigned Addr_RestartEnemyPointsTable2 = 0x0593C3+24;
/* Screens where doors are */
static const unsigned Addr_Door1ScreenTable = 0x059F02;
static const unsigned Addr_Door2ScreenTable = 0x059EF7;

/* Address of the 10-byte table which lists the room numbers
 * of each level where the boss appears.
 */
static const unsigned Addr_BossOffsetTable = 0xCE49;

static const unsigned Addr_BossArrivalInputTable = 0x059380;
static const unsigned Addr_AfterDoorPalette = 0x059F1D;

  // Enemies is list of 4-byte records, terminated by byte $FF.
  //  0: screen #
  //  1: x (center)
  //  2: y (center)
  //  3: type

/* Common ascii characters in all levels */
static const char Ascii_Guts     = 'G';
static const char Ascii_Spike    = '^';
static const char Ascii_Ladder   = '|';
static const char Ascii_WilyDoor = 'D';

static const char Ascii_ObjShutterIn  = 'X'; // input shutter
static const char Ascii_ObjShutterOut = 'S'; // output shutter
// pneumatic  blocks are represented by hex characters 0..9
static const char Ascii_ObjFire    = 'I'; // flying fire

// reserved: X, S, I, 0-9
static const struct ObjectDesc
{
    char ascii;
    unsigned x,y;
} AsciiObjects[] =
{
  //  0
  { 'y', 0x0,0xC }, // bunby heri - the annoying thing in Cutman level beginning
  { 'm', 0x0,0x8 }, // metto
  { 'j', 0x8,0x8 }, // red jumping junk (Flea)
  { 'f', 0x8,0xC }, // floor trap (Spine/Gabyoall)
  //  4
  { 'c', 0x8,0x8 }, // vertically moving eyecube
  { 'v', 0x8,0x0 }, // spread cannon in floor
  { 0 },   // SPECIAL, SEE BELOW
  { 'i', 0x0,0x0 }, // sinewave flying penguin
  //  8
  { 's', 0xF,0x0 }, // sinewave flying bomb
  { 't', 0x0,0xC }, // jumping telephone post (Big Eye) (bomb damage only 4)
  { 'u', 0x8,0x0 }, // hamburger
  { '<', 0x8,0x8 }, // singleshot cannon pointing left (timing 1)
  //  C
  { '-', 0x0,0x8 }, // elec arc
  { 'B', 0x8,0x0 }, // periodical bombing (y = 0xE0) (not destroyable)
  { 0 },   // SPECIAL, SEE BELOW
  { 'I', 0x8,0x0 }, // flying fire, take two! (not destroyable)
  // 10
  { 0 },   // SPECIAL, SEE BELOW
  { 0 },
  { 'G', 0x0,0x4 }, // green guardian (Xl=0, Yl=4)
  { 'L', 0x0,0xC }, // shooting lift ("foot holder") (ont destroyable)
  // 14
  { 'R', 0x0,0x4 }, // robots that may lose their head
  { 'a', 0x0,0x4 }, // axe throwing robot in Gutsman
  { '.', 0x8,0x0 }, // spread cannon pointing down
  { '>', 0x8,0x8 }, // singleshot cannon pointing right
  // 18
  { ']', 0x4,0x6 }, // fire torch to the right (Fireman)
  { '', 0x0,0x7 }, // electricity decoration 1 (Elecman level only)
  { 0 }, // bullet sound? (not destroyable)
  { 0 }, //small explosion (from object $0D but also from enemy killed) (not destroyable)
  // 1C
  { 'r', 0x0,0x4 }, // scissor machine
  { 0 }, //instantiated at $B222, $BC73
  { 0 }, //instantiated at $BD1F
  { 0 }, //instantiated at $B752
  // 20
  { 0 },
  { 'h', 0x8,0x8 }, // horizontally moving eyecube
  { '{', 0x8,0x8 }, // singleshot cannon pointing left (timing 2)
  { '^', 0x8,0x8 }, // singleshot cannon pointing up
  // 24
  { '=', 0x0,0x8 }, // elec arc, different timing
  { '', 0x0,0x7 }, // electricity decoration 2 (Elecman level only)
  { 0 },
  { 0 }, // buzzblocks (Elecman stage)
  // 28
  { 0 }, //instantiated at $AF19
  { 0 }, //instantiated at $B218, $B762
  { 0 }, //instantiated at $AD6A
  { 0 },
  // 2C
  { 0 },   // SPECIAL, SEE BELOW
  { 0 }, //bullet sound? instantiated at $E7F8
  { 0 },
  { 0 },   // SPECIAL, SEE BELOW
  // 30
  { 0 },   // SPECIAL, SEE BELOW
  { 'd', 0x0,0x0 }, // periodically three fire devils from lava to sky
  { 0 },   // SPECIAL, SEE BELOW
  { 0 },
  // 34
  { 0 },
  { 0 },
  { 0 },
  { 0 },
  // 38
  { 0 },
  { 0 },
  { 0 },   // wily3 boss
  { 'T', 0x4,0xC }, // teleport when a "boss" killed, activate enemy if not dead.
  // 3C
  { 'o', 0x8,0x9 }, // score ball = bonus pearl
  { 'p', 0x8,0xC }, // small weapon
  { 'e', 0x8,0x9 }, // small energy
  { 'P', 0x8,0x8 }, // big weapon
  // 40 
  { 'E', 0x8,0x8 }, // big energy
  { 'U', 0x8,0x8 }, // 1up
  { 'M', 0x8,0x8 }, // magnet beam
  { 'O', 0x8,0x0 }, // teleport immediately (invisible)
  // 44
  { '*', 0x8,0x8 }, // end level
  { 'W', 0x8,0x8 }, // all weapons full (yashichi)
  { 0 },
  { 0 },
  // 48
  { 0 },
  { '', 0x8,0x8 }, // random powerup (enemy destruction)
  { 0 }, //? used at 0001CB5B
};

/*
     Controllers:
     
     0x06 functions:
             10,FF = this room has "watchers" (The flying beamers in Elecman and Wily4)
     0x0E functions:
             D0,04 = falling fire (Fireman level)
             B0,FF = zigzag fire (Fireman level)
     0x10 functions:
             E0,80 = activate buzzblocks (Iceman) - used twice
             10,80 = activate buzzblocks (Elecman)
     0x2C functions:
             10,FF = unknown (Wily4, screen 2) - also instantiated by 0001DA9A
     0x2F functions:
             80,80 = super scrolling start (Wily3)
             FC,80 = unknown (Gutsman, screen 1)
             58,80 = unknown (Gutsman, screen 4)
     0x30 functions:
             80,80 = super scrolling stop (Wily3)
             F8,80 = launch lift (Gutsman, screen 0)
             80,FF = stop lift (Gutsman, screen 0)
             60,80 = unknown (Gutsman, screen 5)
     0x32 functions:
             80,E0 = lava activator (Fireman) (??)
     0xFF functions:
             E0,0C = increase pace (wily3)
             01,08 = unknown (Gutsman, screen 5)
             40,0A = unknown (Gutsman, screen 5)
             10,04 = unknown (Gutsman, screen 6)
             50,06 = unknown (Gutsman, screen 6)
             80,02 = water start (Iceman)
             00,00 = water end   (Iceman)
 */


// A level may contain 48 rooms, but when two levels are paired, the total amount must <= 48.
static const unsigned MaxRooms   = 48;
// There is room for 13 actives at max per level.
static const unsigned MaxActives = 32;
// There is room for 419 enemies at max in total.
static const unsigned MaxEnemies = 256;
// There is room for 15 level gfx settings at max per level.
static const unsigned MaxLevelGfx = 15;
// There is room for 11 monster gfx patterns at max per pair.
static const unsigned MaxMonsterGfx = 11;

#endif
